@@ -25,8 +25,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include <the_Foundation/fileinfo.h>
 #include <assert.h>
 
-static_assert(offsetof(iPrefs, plainTextWrap) == offsetof(iPrefs, bools[plainTextWrap_PrefsBool]),
-              "memory layout mismatch (needs struct packing?)");
+_Static_assert(offsetof(iPrefs, plainTextWrap) == offsetof(iPrefs, bools[plainTextWrap_PrefsBool]),
+               "memory layout mismatch (needs struct packing?)");
 
 void init_Prefs(iPrefs *d) {
     iForIndices(i, d->strings) {
