@@ -29,6 +29,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 
 #include "fontpack.h"
 
+iDeclareType(RegExp)
+
 /* Content sizes: regular (1x) -> medium (1.2x) -> big (1.33x) -> large (1.67x) -> huge (2x) */
 
 #define FONT_ID(name, style, size)    ((name) + ((style) * max_FontSize) + (size))
@@ -159,6 +161,7 @@ enum iAnsiFlag {
 void    setOpacity_Text         (float opacity);
 void    setBaseAttributes_Text  (int fontId, int fgColorId); /* current "normal" text attributes */
 void    setAnsiFlags_Text       (int ansiFlags);
+int     ansiFlags_Text          (void);
 
 void    cache_Text              (int fontId, iRangecc text); /* pre-render glyphs */
 
@@ -190,7 +193,9 @@ struct Impl_TextAttrib {
     int16_t fgColorId;
     int16_t bgColorId;
     struct {
+        uint16_t regular   : 1;
         uint16_t bold      : 1;
+        uint16_t light     : 1;
         uint16_t italic    : 1;
         uint16_t monospace : 1;
         uint16_t isBaseRTL : 1;
@@ -202,6 +207,7 @@ struct Impl_WrapText {
     /* arguments */
     iRangecc    text;
     int         maxWidth;
+    size_t      maxLines; /* 0: unlimited */
     enum iWrapTextMode mode;
     iBool     (*wrapFunc)(iWrapText *, iRangecc wrappedText, iTextAttrib attrib, int origin,
                           int advance);
@@ -229,6 +235,8 @@ enum iTextBlockMode { quadrants_TextBlockMode, shading_TextBlockMode };
 iString *   renderBlockChars_Text   (const iBlock *fontData, int height, enum iTextBlockMode,
                                      const iString *text);
 
+iRegExp *   makeAnsiEscapePattern_Text  (void);
+
 /*-----------------------------------------------------------------------------------------------*/
 
 iDeclareType(TextBuf)
@@ -239,6 +247,5 @@ struct Impl_TextBuf {
     iInt2        size;
 };
 
-iTextBuf *  newRange_TextBuf (int font, int color, iRangecc text);
-
+iTextBuf *  newRange_TextBuf(int font, int color, iRangecc text);
 void        draw_TextBuf    (const iTextBuf *, iInt2 pos, int color);
